from azure.storage.blob import ContainerClient
from io import StringIO
import yaml
import pandas as pd


def get_data_using_catalog(dataset_name, config_version, azure_storage_connection_string):
    """Returns a pandas dataframe for a dataset_name stored in a catalog.yml file

    It either gets the data from azure or local

    Filepaths are stored in the catalog.yml according to kedro
    See: https://kedro.readthedocs.io/en/stable/02_get_started/05_example_project.html

    Parameters
    ----------
    dataset_name : str
        The name of the dataset to be loaded according to the catalog.yml-file
    config_version : str
        Which conf-folder to use, according to kedro it can be local or base
    azure_storage_connection_string : str
        Connection string for Azure Data Lake (see Azure Doc. for more info)
    Returns
    -------
    A pandas dataframe
    """

    with open("../conf/{}/catalog.yml".format(config_version), 'r') as stream:
        try:
            yaml_file_content = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)

    # the storage_provider determines if the file is on azure or local
    storage_provider = yaml_file_content.get(dataset_name).get('storage_provider')

    # depending whether it is on azure or local change loading way
    if storage_provider == 'local':
        # get the relative path
        data_file_path = '../' + yaml_file_content.get(dataset_name).get('filepath')
        df = pd.read_csv(data_file_path)
    else:  # it is azure
        data_container_name = yaml_file_content.get(dataset_name).get('container_name')
        data_blob_name = yaml_file_content.get(dataset_name).get('filepath')
        container_client = ContainerClient.from_connection_string(
            conn_str=azure_storage_connection_string
            , container_name=data_container_name
        )
        # Download blob as StorageStreamDownloader object (stored in memory)
        downloaded_blob = container_client.download_blob(data_blob_name)

        df = pd.read_csv(StringIO(downloaded_blob.content_as_text()))

    return df


import pymongo
from pymongo.errors import OperationFailure
from pymongo.uri_parser import parse_uri
import logging
import os


def get_mongo_collection(environment_name, collection_name=False):
    logger = logging.getLogger(__name__)

    env_vars_available = False
    mongodb_connected = False
    db_available = False
    mongo_collection = False

    try:
        mongodb_uri = os.environ['{}_MONGODB_DB_URI'.format(environment_name)]
        mongodb_connect_params = parse_uri(
            mongodb_uri
            , validate=True
            , warn=False
            , normalize=True
            , connect_timeout=None
        )
        mongodb_db_name = mongodb_connect_params.get('database')

        if (mongodb_connect_params.get('collection')) and collection_name:
            logger.error('collection twice available, once in connection string and once passed in')
            return mongo_collection

        if mongodb_connect_params.get('collection'):
            mongodb_collection_name = mongodb_connect_params.get('collection')
            logger.info("loaded mongodb_collection_name from env")
        else:
            mongodb_collection_name = collection_name
            logger.info("loaded mongodb_collection_name from passed in value")
        env_vars_available = True
        logger.info("env_vars_available available")
    except KeyError as e:
        logger.error('issues at os.environ {}'.format(e))

    if env_vars_available:
        try:
            mongo_client = pymongo.MongoClient(mongodb_uri)
            mongodb_connected = True
            logger.info("mongodb_connected")
        except ValueError as e:
            logger.error('issues at mongo_client {}'.format(e))

    if mongodb_connected:
        try:
            mongo_client.list_database_names().index(mongodb_db_name)
            db_available = True
            logger.info("list_database_names available")
        except ValueError as e:
            logger.error('The DB: {} is not available. Error Msg.: {}'.format(mongodb_db_name, e))
        except OperationFailure as e:
            logger.error('cannot authenticate to db'.format(mongodb_db_name, e))

    if db_available:
        # no error handling necessary
        mongo_db = mongo_client[mongodb_db_name]
        mongo_collection = mongo_db[mongodb_collection_name]
        logger.info("opened collection")

    return mongo_collection
